import * as i0 from '@angular/core';
import { Component, Input, EventEmitter, Output, NgModule, ContentChildren, Injectable, Injector } from '@angular/core';
import { CommonModule, DatePipe, DecimalPipe } from '@angular/common';
import { ButtonModule } from 'primeng/button';
import { ProgressSpinnerModule } from 'primeng/progressspinner';
import { RouterModule, Router, ActivatedRoute } from '@angular/router';
import { FormArray, FormsModule, ReactiveFormsModule, FormBuilder } from '@angular/forms';
import { GridCustomTemplate, NgxPrimengGridModule } from 'ngx-primeng-grid';
import { NgxPrimengFormModule, NgxPrimengFormService } from 'ngx-primeng-form';
import { ConfirmDialogModule } from 'primeng/confirmdialog';
import { Subscription, Observable } from 'rxjs';
import { ConfirmationService } from 'primeng/api';
import { DynamicDialogRef, DynamicDialogConfig, DialogService, DynamicDialogModule } from 'primeng/dynamicdialog';
import * as _ from 'lodash';

class NgxInlineLoaderComponent {
    constructor() {
        this.label = 'Loading...';
    }
    ngOnInit() {
    }
}
NgxInlineLoaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-inline-loader',
                template: "<div class=\"text-center\">\r\n    <div>\r\n        <p-progressSpinner></p-progressSpinner>\r\n    </div>\r\n    <strong class=\"d-block\">{{label}}</strong>\r\n</div>",
                styles: [""]
            },] }
];
NgxInlineLoaderComponent.ctorParameters = () => [];
NgxInlineLoaderComponent.propDecorators = {
    label: [{ type: Input }]
};

class NgxSearchComponent {
    constructor() {
        // place holder
        this.placeholder = 'Search';
        // initial value
        this.value = null;
        // on search
        this.onSearch = new EventEmitter();
    }
    ngOnInit() { }
    handleSearch(value) {
        this.onSearch.emit(value);
    }
}
NgxSearchComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-search',
                template: "<div class=\"card-header\">\r\n    <!-- Form -->\r\n    <div class=\"row align-items-center\">\r\n        <div class=\"col\">\r\n            <div class=\"input-group input-group-flush input-group-merge\">\r\n                <!-- Input -->\r\n                <input type=\"search\" class=\"form-control form-control-prepended list-search\" [placeholder]=\"placeholder\"\r\n                    autocomplete=\"off\" (input)=\"handleSearch($event.target.value)\">\r\n                <!-- Prepend -->\r\n                <div class=\"input-group-prepend\">\r\n                    <div class=\"input-group-text\">\r\n                        <span class=\"pi pi-search\"></span>\r\n                    </div>\r\n                </div>\r\n            </div>\r\n        </div>\r\n        <!--Content-->\r\n        <ng-content></ng-content>\r\n    </div>\r\n\r\n</div>",
                styles: [""]
            },] }
];
NgxSearchComponent.propDecorators = {
    placeholder: [{ type: Input }],
    value: [{ type: Input }],
    onSearch: [{ type: Output }]
};

class NgxHeaderComponent {
    constructor() {
        // breadcrumbs
        this.breadcrumbs = [];
    }
    ngOnInit() { }
}
NgxHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-header',
                template: "<div class=\"header\">\r\n    <div class=\"header-body\">\r\n        <div class=\"row align-items-center\">\r\n            <div class=\"col\">\r\n                <h6 class=\"header-pretitle\" *ngIf=\"breadcrumbs && breadcrumbs.length>0\">\r\n                    <ng-container *ngFor=\"let item of breadcrumbs\">\r\n                        <ng-container *ngIf=\"item.path\">\r\n                            <a [routerLink]=\"item.path\">{{item.name}}</a> / \r\n                        </ng-container>\r\n                        <ng-container *ngIf=\"!item.path\">{{item.name}}</ng-container>\r\n                    </ng-container>\r\n                </h6>\r\n                <h1 class=\"header-title\">{{header}}</h1>\r\n            </div>\r\n            <div class=\"col-auto\">\r\n                <ng-content select=\"[actionContent]\"></ng-content>\r\n            </div>\r\n        </div>\r\n        <!--Content-->\r\n        <ng-content></ng-content>\r\n    </div>\r\n</div>",
                styles: [""]
            },] }
];
NgxHeaderComponent.ctorParameters = () => [];
NgxHeaderComponent.propDecorators = {
    header: [{ type: Input }],
    breadcrumbs: [{ type: Input }]
};

class NgxCardLayoutComponent {
    constructor() { }
    ngOnInit() {
    }
}
NgxCardLayoutComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-card-layout',
                template: "<div class=\"card shadow\">\r\n    <div class=\"card-header\">\r\n        <div class=\"row align-items-center\">\r\n            <div class=\"col\">\r\n                <!-- Title -->\r\n                <h4 class=\"card-header-title\"><i class=\"{{headerIcon}}\"></i>{{header}}</h4>\r\n            </div>\r\n            <div class=\"col-auto\">\r\n                <!--Right Content-->\r\n                <ng-content select=\"[rightContent]\"></ng-content>\r\n            </div>\r\n        </div>\r\n    </div>\r\n    <!--content-->\r\n    <ng-content></ng-content>\r\n</div>",
                styles: [""]
            },] }
];
NgxCardLayoutComponent.ctorParameters = () => [];
NgxCardLayoutComponent.propDecorators = {
    header: [{ type: Input }],
    headerIcon: [{ type: Input }]
};

class NgxFullpageLoaderComponent {
    constructor() {
        this.label = 'Loading...';
    }
    ngOnInit() {
    }
}
NgxFullpageLoaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-fullpage-loader',
                template: "<div class=\"full-page-loader\">\n    <div class=\"loader-area\">\n        <ngx-inline-loader [label]=\"label\"></ngx-inline-loader>\n    </div>\n</div>\n",
                styles: [".full-page-loader{height:90vh}.full-page-loader .loader-area{position:absolute;left:50%;top:50%;transform:translate(-50%,-50%)}"]
            },] }
];
NgxFullpageLoaderComponent.ctorParameters = () => [];
NgxFullpageLoaderComponent.propDecorators = {
    label: [{ type: Input }]
};

var ListViewDataType;
(function (ListViewDataType) {
    ListViewDataType["text"] = "text";
    ListViewDataType["number"] = "number";
    ListViewDataType["date"] = "date";
    ListViewDataType["dateUtc"] = "date_utc";
    ListViewDataType["dateTicks"] = "date_ticks";
    ListViewDataType["dateUtcTicks"] = "date_utc_ticks";
})(ListViewDataType || (ListViewDataType = {}));

class NgxListDataViewComponent {
    constructor() {
        this.items = [];
        this._fieldType = ListViewDataType;
    }
    ngOnInit() {
    }
}
NgxListDataViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-list-data-view',
                template: "<div class=\"list-group list-group-flush\">\n    <!--Display Basic Info-->\n    <div class=\"list-group-item\" *ngFor=\"let item of items\">\n        <div class=\"row align-items-center\">\n            <div class=\"col\"><small>{{item.label}}</small></div>\n            <div class=\"col-auto\">\n                <ng-container [ngSwitch]=\"item.type\">\n                    <small class=\"text-muted\"\n                        *ngSwitchCase=\"_fieldType.dateUtc\">{{item.value|date:'mediumDate':'UTC'}}</small>\n                    <small class=\"text-muted\" *ngSwitchCase=\"_fieldType.date\">{{item.value|date:'mediumDate'}}</small>\n                    <small class=\"text-muted\" *ngSwitchDefault>{{item.value}}</small>\n                </ng-container>\n            </div>\n        </div>\n    </div>\n</div>",
                styles: [""]
            },] }
];
NgxListDataViewComponent.ctorParameters = () => [];
NgxListDataViewComponent.propDecorators = {
    items: [{ type: Input }]
};

const components = [
    NgxInlineLoaderComponent,
    NgxSearchComponent,
    NgxHeaderComponent,
    NgxCardLayoutComponent,
    NgxFullpageLoaderComponent,
    NgxListDataViewComponent
];
class NgxWrapperSharedModule {
}
NgxWrapperSharedModule.decorators = [
    { type: NgModule, args: [{
                declarations: [...components],
                imports: [
                    CommonModule,
                    ButtonModule,
                    ProgressSpinnerModule,
                    RouterModule
                ],
                exports: [...components]
            },] }
];

class BaseComponent {
    constructor() {
        // store current user
        this.user = null;
        // define store all subscription
        this.subscription = new Subscription();
    }
    currentUser(key = 'userInfo') {
        if (!this.user) {
            const item = JSON.parse(localStorage.getItem(key));
            if (item != null) {
                this.user = item;
            }
        }
        return this.user;
    }
    // dispose
    dispose() {
        this.subscription.unsubscribe();
    }
}

class NgxManageLayoutComponent extends BaseComponent {
    constructor(router) {
        super();
        this.router = router;
        // store layout
        this._layout = null;
        this.loading = false;
        // fire when user click on add
        this.onAction = new EventEmitter();
        // fire when filter chnaged
        this.onFilter = new EventEmitter();
        // search changed
        this.onSearch = new EventEmitter();
        // breadcrumbs
        this.breadcrumbs = [];
        // on row check
        this.onRowSelected = new EventEmitter();
    }
    set layout(value) {
        if (value) {
            this._layout = value;
        }
    }
    ngOnInit() { }
    ngAfterContentInit() {
        if (this.templates) {
            this.templates.forEach((item) => {
                switch (item.getType()) {
                    case 'custom':
                        {
                            this.customColumnTemplate = item.template;
                        }
                        break;
                }
            });
        }
    }
    // handle when user click on add button
    handleAdd() {
        // check for dialog
        if (this._layout.setting.addOption.mode === 'modal') {
            // oprn 
            this.onAction.emit({ source: 'create' });
        }
        else if (this._layout.setting.addOption.mode === 'link') {
            // link
            this.router.navigateByUrl(this._layout.setting.addOption.link);
        }
        else if (this._layout.setting.addOption.mode === 'custom') {
            // for custom
            this.onAction.emit({ source: 'custom' });
        }
    }
    // handle action for table
    handleAction($instance) {
        this.onAction.emit({ source: 'table', instance: $instance });
    }
    // when table filter changed
    handleFilterChanged($filter) {
        this.onFilter.emit($filter);
    }
    // handle search
    handleSearch(text) {
        this.onSearch.emit(text);
    }
    // handle when row selected by user
    handleOnRowSelected($event) {
        this.onRowSelected.emit($event);
    }
}
NgxManageLayoutComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-manage-layout',
                template: "<!-- Header -->\r\n<ngx-header [header]=\"_layout?.setting?.pageTitle\" [breadcrumbs]=\"breadcrumbs\">\r\n    <div actionContent>\r\n        <!--Add action-->\r\n        <ng-container *ngIf=\"_layout?.setting?.addOption?.enable\">\r\n            <button type=\"button\" pButton pRipple class=\"p-button-sm p-button-outlined\" (click)=\"handleAdd()\"\r\n                [label]=\"_layout?.setting?.addOption?.label\"></button>\r\n        </ng-container>\r\n        <!--action content-->\r\n        <ng-content select=\"[headerActionContent]\"></ng-content>\r\n    </div>\r\n    <!--Header Main Content-->\r\n    <ng-content select=\"[headerMainContent]\"></ng-content>\r\n</ngx-header>\r\n\r\n<!-- Content -->\r\n<div>\r\n    <!--For top content-->\r\n    <ng-content></ng-content>\r\n    <!--Main Content-->\r\n    <div class=\"row\">\r\n        <div class=\"col-12\">\r\n            <!-- Header -->\r\n            <div class=\"card shadow\">\r\n                <!--Search-->\r\n                <ng-container *ngIf=\"_layout?.setting?.enableSearch\">\r\n                    <ngx-search (onSearch)=\"handleSearch($event)\">\r\n                        <ng-content select=\"[searchContent]\"></ng-content>\r\n                    </ngx-search>\r\n                </ng-container>\r\n\r\n                <!--Ngx Table-->\r\n                <ngx-primeng-basic-grid [key]=\"_layout?.setting?.pk\" [columns]=\"_layout?.columns\" [result]=\"result\"\r\n                    [filter]=\"filter\" (onFilterChanged)=\"handleFilterChanged($event)\" (action)=\"handleAction($event)\"\r\n                    (rowChecked)=\"handleOnRowSelected($event)\" [loading]=\"loading\">\r\n\r\n                    <!--Custom column-->\r\n                    <ng-template gridTemplate=\"custom\" let-row=\"row\" let-column=\"column\">\r\n                        <ng-container *ngTemplateOutlet=\"customColumnTemplate;context:{row:row,column:column}\">\r\n                        </ng-container>\r\n                    </ng-template>\r\n\r\n                </ngx-primeng-basic-grid>\r\n            </div>\r\n        </div>\r\n    </div>\r\n    <!--Footer content-->\r\n    <ng-content select=[footer]></ng-content>\r\n</div>",
                styles: [""]
            },] }
];
NgxManageLayoutComponent.ctorParameters = () => [
    { type: Router }
];
NgxManageLayoutComponent.propDecorators = {
    templates: [{ type: ContentChildren, args: [GridCustomTemplate,] }],
    result: [{ type: Input }],
    filter: [{ type: Input }],
    loading: [{ type: Input }],
    layout: [{ type: Input }],
    onAction: [{ type: Output }],
    onFilter: [{ type: Output }],
    onSearch: [{ type: Output }],
    breadcrumbs: [{ type: Input }],
    onRowSelected: [{ type: Output }]
};

class NgxValidationService {
    constructor() { }
    // fire form control validation
    fireFormControlValidation(control) {
        control.markAsTouched();
        control.markAsDirty();
        control.updateValueAndValidity({ emitEvent: true });
    }
    // fire form group validation
    fireFormGroupValidation(formGroup) {
        Object.keys(formGroup.controls).forEach(key => {
            if (formGroup.controls[key] instanceof FormArray) {
                const formsArrayControls = formGroup.controls[key];
                for (const formGroupControl of formsArrayControls.controls) {
                    this.fireFormGroupValidation(formGroupControl);
                }
            }
            // fire form control validation
            this.fireFormControlValidation(formGroup.controls[key]);
        });
    }
    // fire form arra validation
    fireFormArrayValidation(formArray) {
        formArray.controls.forEach((control) => {
            this.fireFormGroupValidation(control);
        });
    }
}
NgxValidationService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgxValidationService_Factory() { return new NgxValidationService(); }, token: NgxValidationService, providedIn: "root" });
NgxValidationService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NgxValidationService.ctorParameters = () => [];

class NgxUtility {
    // clone object
    static clone(obj) {
        return JSON.parse(JSON.stringify(obj));
    }
    static hasProperty(obj, property) {
        return obj && obj.hasOwnProperty(property);
    }
    static hasPropertyValue(obj, property) {
        return obj && obj.hasOwnProperty(property) && obj[property];
    }
    static toSelectItems(items, valueFeild, labelFeild) {
        const result = [];
        if (items && items.length > 0) {
            items.forEach(item => {
                result.push({
                    label: item[labelFeild],
                    value: item[valueFeild]
                });
            });
        }
        return result;
    }
    static toSelectItemsFormat(items, valueFeild, labelFeilds, startSeprator = '<', endSeperator = '>') {
        const result = [];
        if (items && items.length > 0) {
            items.forEach(item => {
                result.push({
                    label: `${labelFeilds[0]}${startSeprator}${labelFeilds[1]}${endSeperator}`,
                    value: item[valueFeild]
                });
            });
        }
        return result;
    }
    static scrollToTop() {
        window.scrollTo({
            top: 0,
            behavior: 'smooth'
        });
    }
    static scrollTo(elementId) {
        const element = document.getElementById(elementId);
        if (element) {
            element.scrollIntoView({
                behavior: 'smooth'
            });
        }
        ;
    }
    static toFloat(value, defValue = 0) {
        const v = parseFloat(value);
        if (isNaN(v)) {
            return defValue;
        }
        return v;
    }
    static toFloatExact(value) {
        return NgxUtility.toFloat(value.toFixed(10));
    }
    static toInt(value, defValue = 0) {
        const v = parseInt(value, 10);
        if (isNaN(v)) {
            return defValue;
        }
        return v;
    }
    static trim(obj) {
        if (obj) {
            Object.keys(obj).map(k => obj[k] = typeof obj[k] == 'string' ? obj[k].trim() : obj[k]);
        }
    }
    static getUniqueString() {
        return `${new Date().getTime()}_${Math.floor((Math.random() * 9999999) + 1)}`;
    }
    static jsonToDate(jsonStr) {
        if (jsonStr) {
            // get dt object
            const dt = new Date(jsonStr);
            //NUll check
            return NgxUtility.isValidDate(dt) ? dt : null;
        }
        return null;
    }
    static isValidDate(date) {
        //NUll check
        if (date) {
            return isNaN(date.getTime()) ? false : true;
        }
    }
    static localToUtc(date) {
        if (NgxUtility.isValidDate(date)) {
            return new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds()));
        }
        return;
    }
    static utcToLocal(date) {
        if (NgxUtility.isValidDate(date)) {
            return new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds());
        }
        return null;
    }
    static toDateFormat(dtStr, format, timeZone = undefined) {
        // get date
        const dt = NgxUtility.jsonToDate(dtStr);
        if (dt) {
            const datePipe = new DatePipe('en-US');
            return datePipe.transform(dt, format, timeZone);
        }
        return null;
    }
    static toAmountFormat(amount, currency = '') {
        if (amount) {
            const decimalPipe = new DecimalPipe('en-US');
            return `${currency}${decimalPipe.transform(amount, '1.2-2')}`;
        }
        return null;
    }
    static focusElement(elementId) {
        const ele = document.getElementById(elementId);
        if (ele) {
            ele.focus();
            ele.select();
        }
    }
    static dialogContentStyle(maxHeight = '80vh') {
        return { "max-height": maxHeight, "overflow-y": "auto", "overflow-x": "hidden" };
    }
    static datePickerToDate(value, splitChar = '-') {
        if (value) {
            const values = value.split(splitChar);
            // dd-mm-yy - 0-1-2
            return new Date(NgxUtility.toInt(values[2]), NgxUtility.toInt(values[1]) - 1, NgxUtility.toInt(values[0]));
        }
        return null;
    }
    static monthPickerToDate(value, splitChar = '-') {
        if (value) {
            const values = value.split(splitChar);
            // dd-mm-yy - 0-1-2
            return new Date(NgxUtility.toInt(values[2]), NgxUtility.toInt(values[1]) - 1, 1);
        }
        return null;
    }
    static isSuccessResult(result) {
        return NgxUtility.toInt(result.status) === 1 ? true : false;
    }
    static patchFormValue(value, formGroup) {
        if (value) {
            Object.keys(formGroup.controls).forEach(controlName => {
                if (value.hasOwnProperty(controlName)) {
                    const data = value[controlName];
                    formGroup.controls[controlName].setValue(data);
                }
            });
        }
    }
    static cleanNullableObject(obj) {
        Object.keys(obj).forEach(key => {
            if (obj[key] && typeof obj[key] === "object")
                NgxUtility.cleanNullableObject(obj[key]); // recurse
            else if (obj[key] == null)
                delete obj[key]; // delete
        });
    }
    static getRouteKey(key, activatedRoute) {
        if (activatedRoute.snapshot.paramMap.has(key)) {
            return activatedRoute.snapshot.paramMap.get(key);
        }
        else {
            let value = null;
            let parent = activatedRoute.parent;
            let foundKey = false;
            while (!foundKey) {
                if (parent) {
                    if (parent.snapshot.paramMap.has(key)) {
                        value = parent.snapshot.paramMap.get(key);
                        foundKey = true;
                    }
                    else {
                        parent = parent.parent;
                    }
                }
                else {
                    foundKey = true;
                }
            }
            return value;
        }
    }
    static getRouteData(key, activatedRoute) {
        if (NgxUtility.hasProperty(activatedRoute.snapshot.data, key)) {
            return activatedRoute.snapshot.data[key];
        }
        else {
            let value = null;
            let parent = activatedRoute.parent;
            let foundKey = false;
            while (!foundKey) {
                if (parent) {
                    if (NgxUtility.hasProperty(parent.snapshot.data, key)) {
                        value = parent.snapshot.data[key];
                        foundKey = true;
                    }
                    else {
                        parent = parent.parent;
                    }
                }
                else {
                    foundKey = true;
                }
            }
            return value;
        }
    }
}

class NgxManageActionLayoutComponent {
    constructor(validationService, confirmDialog) {
        this.validationService = validationService;
        this.confirmDialog = confirmDialog;
        // action items
        this.actions = [];
        // confirmations
        this.confirmations = [];
        // on submit
        this.onAction = new EventEmitter();
    }
    ngOnInit() {
        // init dialog key
        this.dialogKey = `action_layout_cd_${NgxUtility.getUniqueString()}`;
        // check there is no actions
        if (this.actions.length == 0) {
            this.actions.push({ key: 'submit', name: 'Submit', styleClass: 'p-button-sm p-button-outlined mr-2', validate: true });
            this.actions.push({ key: 'cancel', name: 'Cancel', styleClass: 'p-button-sm p-button-outlined p-button-danger', validate: false });
        }
    }
    handleAction(item) {
        if (item.validate) {
            if (this.form.valid) {
                // check confirm action
                this.checkConfirmation(item, () => {
                    this.onAction.emit(item);
                });
            }
            else {
                this.validationService.fireFormGroupValidation(this.form);
            }
        }
        else {
            // check confirm action
            this.checkConfirmation(item, () => {
                this.onAction.emit(item);
            });
        }
    }
    checkConfirmation(action, callback) {
        if (this.confirmations && this.confirmations.length > 0) {
            // get the confirm info
            const confirmInfo = this.confirmations.find(m => m.key == action.key);
            if (confirmInfo) {
                this.confirmDialog.confirm({
                    header: confirmInfo.heading ? confirmInfo.heading : 'Confirmation',
                    message: confirmInfo.message,
                    key: this.dialogKey,
                    icon: 'pi pi-exclamation-triangle',
                    accept: () => {
                        callback();
                    }
                });
            }
            else {
                callback();
            }
        }
        else {
            callback();
        }
    }
}
NgxManageActionLayoutComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-manage-action-layout',
                template: "<form [formGroup]=\"form\" autocomplete=\"off\">\r\n    <!--Content-->\r\n    <div>\r\n        <ng-content></ng-content>\r\n    </div>\r\n    <!--Action-->\r\n    <div class=\"d-flex justify-content-end\">\r\n        <hr class=\"m-2\" />\r\n        <button *ngFor=\"let item of actions\" type=\"button\" pButton pRipple class=\"{{item.styleClass}}\"\r\n            (click)=\"handleAction(item)\" [label]=\"item.name\"></button>\r\n    </div>\r\n</form>\r\n<!--Confirm dialog-->\r\n<p-confirmDialog appendTo=\"body\" [key]=\"dialogKey\"></p-confirmDialog>",
                styles: [""]
            },] }
];
NgxManageActionLayoutComponent.ctorParameters = () => [
    { type: NgxValidationService },
    { type: ConfirmationService }
];
NgxManageActionLayoutComponent.propDecorators = {
    actions: [{ type: Input }],
    confirmations: [{ type: Input }],
    form: [{ type: Input }],
    onAction: [{ type: Output }]
};

class NgxManageLayoutModule {
}
NgxManageLayoutModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgxManageLayoutComponent, NgxManageActionLayoutComponent],
                imports: [
                    CommonModule,
                    FormsModule,
                    ConfirmDialogModule,
                    ReactiveFormsModule,
                    NgxPrimengFormModule,
                    NgxPrimengGridModule,
                    NgxWrapperSharedModule,
                    ButtonModule,
                    RouterModule
                ],
                exports: [NgxManageLayoutComponent, NgxManageActionLayoutComponent]
            },] }
];

const modules = [
    NgxWrapperSharedModule,
    NgxManageLayoutModule
];
class NgxWrapperModule {
}
NgxWrapperModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    CommonModule,
                    ...modules
                ],
                exports: [...modules]
            },] }
];

class DialogBaseComponent extends BaseComponent {
    constructor(dialogFormInjector) {
        super();
        // prepare instance
        this.dialogBaseRef = dialogFormInjector.get(DynamicDialogRef);
        this.dialogBaseConfig = dialogFormInjector.get(DynamicDialogConfig);
    }
    // get dialog data
    get dialogData() {
        const data = this.dialogBaseConfig.data;
        return data ? data : null;
    }
    // close dialog
    close(result = null) {
        this.dialogBaseRef.close(result);
    }
}

class NgxLambdaUtility {
    // get member name from expression
    // !!Experimental
    static getMemberNameFromExpression(callbackfn) {
        const methodBody = callbackfn.substring(callbackfn.indexOf('{') + 1, callbackfn.lastIndexOf('}'));
        // get only last part
        const splitItems = methodBody.split('.');
        // check length of split item
        if (splitItems.length === 2) {
            // last item and remove ;, which is added by method
            return splitItems[1].replace(';', '').trim();
        }
        else if (splitItems.length === 3) {
            // last item and remove ;, which is added by method
            return splitItems[2].replace(';', '').trim();
        }
        return null;
    }
    // to select items
    static toSelectItems(items, callbackfnValue, callbackfnLabel) {
        const valueField = this.getMemberNameFromExpression(callbackfnValue.toString());
        const labelField = this.getMemberNameFromExpression(callbackfnLabel.toString());
        return NgxUtility.toSelectItems(items, valueField, labelField);
    }
}

class FormBaseComponent extends BaseComponent {
    constructor(formInjector) {
        super();
        this.formBaseValidationService = formInjector.get(NgxValidationService);
    }
    getValue(controlName, formGroup = this.form) {
        return formGroup.controls[controlName].value;
    }
    setValue(controlName, data, formGroup = this.form) {
        return formGroup.controls[controlName].setValue(data);
    }
    patchValue(value, formGroup = this.form) {
        NgxUtility.patchFormValue(value, formGroup);
    }
    getControl(controlName, formGroup = this.form) {
        return formGroup.controls[controlName];
    }
    // get the form value from expression
    // !! Expermiental
    getFormValue(callbackfn, formGroup = this.form) {
        const memberName = NgxLambdaUtility.getMemberNameFromExpression(callbackfn.toString());
        return this.getValue(memberName, formGroup);
    }
    // set the form value from expression
    // !! Expermiental
    setFormValue(callbackfn, data, formGroup = this.form) {
        const memberName = NgxLambdaUtility.getMemberNameFromExpression(callbackfn.toString());
        this.setValue(memberName, data, formGroup);
    }
    // get control by expression
    // !! Expermiental
    getFormControl(callbackfn, formGroup = this.form) {
        const memberName = NgxLambdaUtility.getMemberNameFromExpression(callbackfn.toString());
        return this.getControl(memberName, formGroup);
    }
    cloneFormGroup(copyForm, copyTo, clearAll = true) {
        if (clearAll) {
            Object.keys(copyTo.controls).forEach(controlName => {
                copyTo.removeControl(controlName);
            });
        }
        // copy value
        Object.keys(copyForm.controls).forEach(controlName => {
            copyTo.addControl(controlName, copyForm.controls[controlName]);
        });
    }
    // fire control validation
    fireFormControlValidation(control) {
        this.formBaseValidationService.fireFormControlValidation(control);
    }
    // fire form group validation
    fireFormGroupValidation(formGroup = this.form) {
        this.formBaseValidationService.fireFormGroupValidation(formGroup);
    }
    // fire form arra validation
    fireFormArrayValidation(formArray) {
        this.formBaseValidationService.fireFormArrayValidation(formArray);
    }
}

class DynamicFormBaseComponent extends FormBaseComponent {
    constructor(dynamicFormInjector) {
        super(dynamicFormInjector);
        // prepare instance
        this.ngxPrimengFormService = dynamicFormInjector.get(NgxPrimengFormService);
    }
    // prepare form
    prepareForm(formBuilder, jsonForms) {
        return this.ngxPrimengFormService.prepareForm(formBuilder, jsonForms);
    }
    // load dynamic forms
    loadRemoteJsonForm(formBuilder, formObservable) {
        const observable = new Observable((observer) => {
            const ref = formObservable.subscribe(forms => {
                // load form data
                const result = this.prepareForm(formBuilder, forms);
                // emit event
                observer.next(result);
                // complete
                observer.complete();
            }, error => {
                observer.error(error);
            });
            this.subscription.add(ref);
        });
        return observable;
    }
}

class DialogFormBaseComponent extends DynamicFormBaseComponent {
    constructor(dialogFormInjector) {
        super(dialogFormInjector);
        // prepare instance
        this.dialogBaseRef = dialogFormInjector.get(DynamicDialogRef);
        this.dialogBaseConfig = dialogFormInjector.get(DynamicDialogConfig);
    }
    // get dialog data
    get dialogData() {
        const data = this.dialogBaseConfig.data;
        return data ? data : null;
    }
    // close dialog
    closeDialog(result = null) {
        this.dialogBaseRef.close(result);
    }
}

class NgxManageLayoutService {
    constructor() { }
    process(manageJson) {
        if (manageJson) {
            const obj = manageJson;
            const result = {
                api: obj.api,
                columns: this.processColumn(obj.columns),
                setting: this.processSetting(obj.setting),
                form: this.processForm(obj.form)
            };
            return result;
        }
        return null;
    }
    processColumn(columns) {
        if (columns && columns.length > 0) {
        }
        return columns;
    }
    processSetting(setting) {
        return {
            pageTitle: setting.pageTitle,
            addOption: this.getAddOption(setting.addOption),
            enableSearch: NgxUtility.hasProperty(setting, 'enableSearch') ? setting.enableSearch : true,
            perPage: NgxUtility.hasProperty(setting, 'perPage') ? setting.perPage : 10,
            pk: setting.pk,
            layoutId: NgxUtility.hasProperty(setting, 'layoutId') ? setting.layoutId : null,
            saveSearch: NgxUtility.hasProperty(setting, 'saveSearch') ? setting.saveSearch : true,
            actionMode: NgxUtility.hasProperty(setting, 'actionMode') ? setting.actionMode : 'block',
            defaultSortColumn: NgxUtility.hasProperty(setting, 'defaultSortColumn') ? setting.defaultSortColumn : null,
            defaultSortOrder: NgxUtility.hasProperty(setting, 'defaultSortOrder') ? setting.defaultSortOrder : -1,
        };
    }
    getAddOption(option) {
        if (option) {
            return {
                enable: NgxUtility.hasProperty(option, 'enable') ? option.enable : false,
                label: NgxUtility.hasProperty(option, 'label') ? option.label : null,
                mode: NgxUtility.hasProperty(option, 'mode') ? option.mode : null,
                link: NgxUtility.hasProperty(option, 'link') ? option.link : null,
                modalTitle: NgxUtility.hasProperty(option, 'modalTitle') ? option.modalTitle : null,
                modalWidth: NgxUtility.hasProperty(option, 'modalWidth') ? option.modalWidth : 'auto',
            };
        }
        return null;
    }
    processForm(form) {
        return {
            actions: NgxUtility.hasProperty(form, 'actions') ? form.actions : [],
            controls: NgxUtility.hasProperty(form, 'controls') ? form.controls : [],
            confirmations: NgxUtility.hasProperty(form, 'confirmations') ? form.confirmations : [],
        };
    }
}
NgxManageLayoutService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgxManageLayoutService_Factory() { return new NgxManageLayoutService(); }, token: NgxManageLayoutService, providedIn: "root" });
NgxManageLayoutService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NgxManageLayoutService.ctorParameters = () => [];

class NgxManageLayoutBaseComponent extends FormBaseComponent {
    // get manage component
    constructor(manageInjector) {
        super(manageInjector);
        this.loading = false;
        // columns
        this.columns = [];
        // result
        this.result = { results: [], total: 0 };
        // breadcrumbs
        this.breadcrumbs = [];
        // selected rows
        this.selectedRows = [];
        // init instance
        this.manageDialog = manageInjector.get(DialogService);
        this.ngxLayoutBaseService = manageInjector.get(NgxManageLayoutService);
        this.activatedBaseRoute = manageInjector.get(ActivatedRoute);
    }
    // prepare layout from router
    prepareLayoutFromRouter(invokeSearch = true, dataKey = 'layout') {
        const data = this.activatedBaseRoute.snapshot.data;
        if (NgxUtility.hasPropertyValue(data, dataKey)) {
            const manageJson = data[dataKey];
            // prepare layout
            this.prepareLayout(manageJson, invokeSearch);
        }
    }
    // prepare layout from service
    prepareLayoutFromService(jsonObjectService, successCallback, invokeSearch = true) {
        const ref = jsonObjectService.subscribe(jsonObject => {
            // on success result
            successCallback(jsonObject);
            // prepare layout
            this.prepareLayout(jsonObject, invokeSearch);
        });
        this.subscription.add(ref);
    }
    // prepare layout from json
    prepareLayout(jsonObject, invokeSearch) {
        if (!jsonObject) {
            return;
        }
        // get the title
        this.layout = this.ngxLayoutBaseService.process(jsonObject);
        // prepare columns
        this.columns = this.layout.columns;
        //override layout
        this.overrideLayout(this.layout);
        // init filter
        this.initFilter(this.layout.setting);
        // override Filter
        this.overrideFilter(this.filter);
        // do search
        if (invokeSearch) {
            this.doSearch();
        }
    }
    // handle action
    handleOnAction($action) {
        if ($action.source === 'create') {
            this.openManageDialog(null);
        }
        else if ($action.source === 'table') {
            this.handleTableAction($action.instance);
        }
        else if ($action.source === 'custom') {
            this.overrideAction(this.layout.setting.addOption);
        }
    }
    // handle when filter changed
    handleOnFilter($filter) {
        const reqFilter = _.cloneDeep($filter);
        // only set basic filter from grid
        this.filter.pageNo = reqFilter.pageNo;
        this.filter.perPage = reqFilter.perPage;
        this.filter.sortField = reqFilter.sortField;
        this.filter.sortOrder = reqFilter.sortOrder;
        this.doSearch();
    }
    // init filter
    initFilter(setting) {
        this.filter = {
            pageNo: 0,
            perPage: setting.perPage,
            sortField: setting.defaultSortColumn,
            sortOrder: setting.defaultSortOrder,
            formFilter: null,
            searchText: null
        };
    }
    // handle when search changed
    handleOnSearch(text) {
        // on search reset the page no
        this.filter.pageNo = 0;
        this.filter.searchText = text;
        this.doSearch();
    }
    // handle row selected
    handleOnRowSelected($event) {
        this.selectedRows = $event;
    }
    handleTableAction(instance) {
        switch (instance.column.key) {
            case 'edit':
                {
                    this.openManageDialog(instance.row);
                }
                break;
            case 'delete':
                {
                    this.handleDelete(instance.row);
                }
                break;
            case 'status':
                {
                    this.handleStatusChange(instance.row);
                }
                break;
            default:
                {
                    this.handleRowAction(instance);
                }
                break;
        }
    }
    openManageDialog(model) {
        // data model
        const dataModel = {
            model,
            layout: this.layout,
            data: this.getExtraDialogData()
        };
        // dialog
        const dialogRef = this.manageDialog.open(this.getManageDialogComponent(), {
            header: this.layout.setting.addOption.modalTitle,
            width: this.layout.setting.addOption.modalWidth,
            contentStyle: NgxUtility.dialogContentStyle(),
            data: dataModel
        });
        // handle close event
        const ref = dialogRef.onClose.subscribe(result => {
            // unsubscribe
            ref.unsubscribe();
            // do work
            if (result) {
                this.doSearch();
            }
        });
    }
    handleDelete(model) {
        // pre action
        this.overrideHttpCallPreAction(model);
        const ref = this.getDeleteService(model).subscribe(res => {
            // post action
            this.overrideHttpCallPostAction(res);
            // search
            this.doSearch();
        });
        this.subscription.add(ref);
    }
    handleStatusChange(model) {
        // pre action
        this.overrideHttpCallPreAction(model);
        const ref = this.getChangeStatusService(model).subscribe(res => {
            // post action
            this.overrideHttpCallPostAction(res);
        });
        this.subscription.add(ref);
    }
    // do search
    doSearch() {
        this.loading = true;
        const ref = this.getSearchService(this.filter).subscribe(res => {
            this.result = res;
            // reset the selected rows
            this.selectedRows = [];
            // set loading false
            this.loading = false;
        });
        this.subscription.add(ref);
    }
    //  override Layout
    overrideLayout($layout) { }
    // override Filter
    overrideFilter($filter) { }
    // override Action for custom
    overrideAction($option) { }
    // override http call pre action
    overrideHttpCallPreAction(model) { }
    // override http call pre action
    overrideHttpCallPostAction(result) { }
}

class NgxManageLayoutDialogBaseComponent extends DialogFormBaseComponent {
    constructor(manageDialognjector) {
        super(manageDialognjector);
        // define form items
        this.formItems = [];
    }
    // load form
    loadForm(formBuilder) {
        // make sure form shoule initialize
        const formResult = this.prepareForm(formBuilder, this.dialogData.layout.form.controls);
        // clone form
        this.cloneFormGroup(formResult.formGroup, this.form);
        // set form items
        this.formItems = formResult.forms;
        // process form data
        this.processData(this.form);
    }
    // process data
    processData(formGroup) {
        const data = this.dialogData;
        if (data && data.model) {
            //pre action
            // clone
            const model = _.cloneDeep(data.model);
            // patach value
            this.patchValue(this.overrideSetFormDataPreAction(model), formGroup);
            // post action
            this.overrideSetFormDataPostAction();
        }
    }
    // handle when action performed
    handleAction(event) {
        switch (event.key) {
            case 'cancel':
                {
                    this.closeDialog();
                }
                break;
            default:
                {
                    this.handleOnAction(this.getActionFormValue(), event);
                }
                break;
        }
    }
    // get action form value
    getActionFormValue() {
        // get form value
        const formValue = _.cloneDeep(this.form.value);
        // get dialog data
        const data = _.cloneDeep(this.dialogData);
        // null check
        if (data && data.model) {
            // set primary key
            // cross check that model has key or not
            if (NgxUtility.hasPropertyValue(data.model, this.dialogData.layout.setting.pk)) {
                formValue[this.dialogData.layout.setting.pk] = this.dialogData.model[this.dialogData.layout.setting.pk];
            }
        }
        return formValue;
    }
    overrideSetFormDataPreAction(model) { return model; }
    overrideSetFormDataPostAction() { }
}

;

class NgxCrudManageComponent extends DynamicFormBaseComponent {
    constructor(injector, formBuilder) {
        super(injector);
        this.formBuilder = formBuilder;
        // define form items
        this._formItems = [];
        // action 
        this.onAction = new EventEmitter();
        // init form
        this.form = this.formBuilder.group({});
    }
    ngOnInit() {
        // load form
        this.loadForm();
    }
    // load form
    loadForm() {
        if (this.layout) {
            // make sure form shoule initialize
            this.layout.form['controlResult'] = this.prepareForm(this.formBuilder, this.layout.form.controls);
            // clone form
            this.cloneFormGroup(this.layout.form.controlResult.formGroup, this.form);
            // set form items
            this._formItems = this.layout.form.controlResult.forms;
            // process patch value
            if (this.data) {
                this.layout.form.action.onPatchValue(this.form, this._formItems, this.data);
            }
            // process prepare value
            this.layout.form.action.onPrepareValue(this.form, this._formItems, this.data);
        }
    }
    handleOnAction(action) {
        // service call
        if (action.serviceCall) {
            // get form value
            const formValue = _.cloneDeep(this.form.getRawValue());
            // execute command
            this.layout.form.action.onExecute(formValue, action, this.data, result => {
                if (NgxUtility.isSuccessResult(result)) {
                    // call back
                    this.onAction.emit({ action, result });
                }
            });
        }
        else {
            this.onAction.emit({ action });
        }
    }
    ngOnDestroy() {
        this.dispose();
    }
}
NgxCrudManageComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-crud-manage',
                template: "<ngx-crud-manage-layout [form]=\"form\" (onAction)=\"handleOnAction($event)\" [actions]=\"layout?.form?.actions\">\n    <ngx-primeng-form [form]=\"form\" [items]=\"_formItems\"></ngx-primeng-form>\n</ngx-crud-manage-layout>",
                styles: [""]
            },] }
];
NgxCrudManageComponent.ctorParameters = () => [
    { type: Injector },
    { type: FormBuilder }
];
NgxCrudManageComponent.propDecorators = {
    layout: [{ type: Input }],
    onAction: [{ type: Output }],
    data: [{ type: Input }]
};

class NgxCrudManageDialogComponent extends DialogFormBaseComponent {
    constructor(injector) {
        super(injector);
    }
    ngOnInit() {
        // layout
        this._layout = this.dialogData.layout;
        // modal
        this._data = this.dialogData.model;
        // patch value
        this._layout.form.action.onPatchValue = (formGroup, formItems, data) => {
            // patch value
            this.patchValue(data, formGroup);
        };
    }
    handleOnAction(callback) {
        this.closeDialog(callback.action.serviceCall ? callback.result : null);
    }
    ngOnDestroy() {
        this.dispose();
    }
}
NgxCrudManageDialogComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-crud-manage-dialog',
                template: "<ngx-crud-manage [layout]=\"_layout\" (onAction)=\"handleOnAction($event)\" [data]=\"_data\"></ngx-crud-manage>",
                styles: [""]
            },] }
];
NgxCrudManageDialogComponent.ctorParameters = () => [
    { type: Injector }
];

class NgxCrudManageLayoutComponent extends FormBaseComponent {
    constructor(injector, confirmDialog) {
        super(injector);
        this.confirmDialog = confirmDialog;
        // actions
        this.actions = [];
        // on action perfomed
        this.onAction = new EventEmitter();
    }
    ngOnInit() {
        // init dialog key
        this._dialogKey = `crud_action_layout_cd_${new Date().getTime()}`;
        // prepare default actions
        this.prepareDefaultActions();
    }
    prepareDefaultActions() {
        if (!this.actions) {
            this.actions = [];
            this.actions.push({ key: 'cancel', name: 'Cancel', styleClass: 'p-button-sm p-button-outlined p-button-danger mr-2', validate: false, serviceCall: false });
            this.actions.push({ key: 'submit', name: 'Submit', styleClass: 'p-button-sm p-button-outlined', validate: true, serviceCall: true });
        }
    }
    handleAction(item) {
        if (item.validate) {
            if (this.form.valid) {
                // check confirm action
                this.checkConfirmation(item, () => {
                    this.onAction.emit(item);
                });
            }
            else {
                this.fireFormGroupValidation(this.form);
            }
        }
        else {
            // check confirm action
            this.checkConfirmation(item, () => {
                this.onAction.emit(item);
            });
        }
    }
    checkConfirmation(action, callback) {
        if (action.confirmMessage) {
            this.confirmDialog.confirm({
                header: 'Confirmation',
                message: action.confirmMessage,
                key: this._dialogKey,
                icon: 'pi pi-exclamation-triangle',
                accept: () => {
                    callback();
                }
            });
        }
        else {
            callback();
        }
    }
}
NgxCrudManageLayoutComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-crud-manage-layout',
                template: "<form [formGroup]=\"form\" autocomplete=\"off\">\n    <!--Content-->\n    <div>\n        <ng-content></ng-content>\n    </div>\n    <!--Action-->\n    <hr class=\"m-3\" />\n    <div class=\"d-flex justify-content-end\">\n        <button *ngFor=\"let item of actions\" type=\"button\" pButton pRipple class=\"{{item.styleClass}}\"\n            (click)=\"handleAction(item)\" [label]=\"item.name\"></button>\n    </div>\n</form>\n<!--Confirm dialog-->\n<p-confirmDialog appendTo=\"body\" [key]=\"_dialogKey\"></p-confirmDialog>",
                styles: [""]
            },] }
];
NgxCrudManageLayoutComponent.ctorParameters = () => [
    { type: Injector },
    { type: ConfirmationService }
];
NgxCrudManageLayoutComponent.propDecorators = {
    actions: [{ type: Input }],
    form: [{ type: Input }],
    onAction: [{ type: Output }]
};

var NgxCrudLayoutSetting_ManageConfig_Mode;
(function (NgxCrudLayoutSetting_ManageConfig_Mode) {
    NgxCrudLayoutSetting_ManageConfig_Mode["modal"] = "modal";
    NgxCrudLayoutSetting_ManageConfig_Mode["custom"] = "custom";
})(NgxCrudLayoutSetting_ManageConfig_Mode || (NgxCrudLayoutSetting_ManageConfig_Mode = {}));
;

class NgxCrudGridComponent extends BaseComponent {
    constructor(dialog, confirmDialog) {
        super();
        this.dialog = dialog;
        this.confirmDialog = confirmDialog;
        // loading indicator
        this._loading = true;
        // breadcrumbs
        this.breadcrumbs = [];
        // on grid ready
        this.onReady = new EventEmitter();
        // result
        this._result = { results: [], total: 0 };
    }
    set layout(value) {
        if (value) {
            this._layout = value;
        }
    }
    ngOnInit() {
        // init dialog key
        this._dialogKey = `crud_grid_layout_cd_${NgxUtility.getUniqueString()}`;
        // inti filter
        this.initFilter();
    }
    ngAfterContentInit() {
        if (this.templates) {
            this.templates.forEach((item) => {
                switch (item.getType()) {
                    case 'custom':
                        {
                            this._customColumnTemplate = item.template;
                        }
                        break;
                }
            });
        }
    }
    // handle when user click on add button
    handleActionClick(action) {
        // check for type
        if (action.mode === NgxCrudLayoutSetting_ManageConfig_Mode.modal) {
            this.openManageDialog({ layout: this._layout, model: null });
        }
        else {
            // fire action
            this._layout.setting.actionConfig.onAction(action);
        }
    }
    // get action dialog from the setting
    getModalActionAction() {
        const actionConfig = this._layout.setting.actionConfig;
        return actionConfig.actions.find(m => m.mode == NgxCrudLayoutSetting_ManageConfig_Mode.modal);
    }
    // open manage dialog
    openManageDialog(data) {
        // get action config
        const action = this.getModalActionAction();
        if (action) {
            const dialogRef = this.dialog.open(NgxCrudManageDialogComponent, {
                header: action.modalConfig.title,
                width: action.modalConfig.width,
                contentStyle: NgxUtility.dialogContentStyle(),
                data
            });
            const ref1 = dialogRef.onClose.subscribe(result => {
                ref1.unsubscribe();
                if (result) {
                    // search result
                    this.lodResult();
                }
            });
        }
        else {
            throw 'Invalid Modal Action';
        }
    }
    // when table filter changed
    handleFilterChanged(filter) {
        // set filter
        const reqFilter = _.cloneDeep(filter);
        // only set basic filter from grid
        this._layout.setting.gridConfig.action.filter.pageNo = reqFilter.pageNo;
        this._layout.setting.gridConfig.action.filter.perPage = reqFilter.perPage;
        this._layout.setting.gridConfig.action.filter.sortField = reqFilter.sortField;
        this._layout.setting.gridConfig.action.filter.sortOrder = reqFilter.sortOrder;
        // fire filter
        this._layout.setting.gridConfig.action.onFilterChanged(this._layout.setting.gridConfig.action.filter);
        // fire load result
        this.lodResult();
    }
    // handle search
    handleSearch(text) {
        // set filter
        this._layout.setting.gridConfig.action.filter.pageNo = 0;
        this._layout.setting.gridConfig.action.filter.searchText = text;
        // fire load result
        this.lodResult();
    }
    // for edit action
    onActionEdit(row) {
        const modalAction = this.getModalActionAction();
        if (modalAction) {
            this.openManageDialog({ model: row, layout: this._layout });
        }
        else {
            this.invokeGridAction('edit', row);
        }
    }
    // for delete action
    onActionDelete(row) {
        this.confirmDialog.confirm({
            header: 'Confirmation',
            message: 'Are you sure that you want to delete?',
            key: this._dialogKey,
            icon: 'pi pi-exclamation-triangle',
            accept: () => {
                this.invokeGridAction('delete', row);
            }
        });
    }
    // invoke grid action
    invokeGridAction(actionName, data) {
        if (this._layout.setting.gridConfig.action.customAction.invoke) {
            this._layout.setting.gridConfig.action.customAction.invoke(actionName, data);
        }
    }
    // init filter
    initFilter() {
        this._filter = {
            pageNo: 0,
            perPage: this._layout.setting.gridConfig.perPage,
            sortField: this._layout.setting.gridConfig.sortField,
            sortOrder: this._layout.setting.gridConfig.sortOrder,
            formFilter: null,
            searchText: null
        };
        // set filter
        this._layout.setting.gridConfig.action['filter'] = this._filter;
        // call on ready
        this.onReady.emit(this._layout);
        // load result
        this.lodResult();
    }
    // load results
    lodResult() {
        this._loading = true;
        this._layout.setting.gridConfig.action.loadResult(result => {
            // set result
            this._result = result;
            // hide loading
            this._loading = false;
        });
    }
    ngOnDestroy() {
        this.dispose();
    }
}
NgxCrudGridComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-crud-grid',
                template: "<!-- Header -->\n<ngx-header [header]=\"_layout?.setting?.pageTitle\" [breadcrumbs]=\"_layout?.setting?.gridConfig?.breadcrumbs\">\n    <div actionContent>\n        <!--manage action-->\n        <ng-container *ngFor=\"let action of _layout?.setting?.actionConfig?.actions\">\n            <button type=\"button\" pButton pRipple [class]=\"action?.styleClass\" (click)=\"handleActionClick(action)\"\n                [label]=\"action?.label\" [icon]=\"action?.icon\"></button>\n        </ng-container>\n        <!--action content-->\n        <ng-content select=\"[headerActionContent]\"></ng-content>\n    </div>\n    <!--Header Main Content-->\n    <ng-content select=\"[headerMainContent]\"></ng-content>\n</ngx-header>\n\n<!-- Content -->\n<div>\n    <!--For top content-->\n    <ng-content></ng-content>\n    <!--Main Content-->\n    <div class=\"row\">\n        <div class=\"col-12\">\n            <!-- Header -->\n            <div class=\"card shadow\">\n                <!--Search-->\n                <ng-container *ngIf=\"_layout?.setting?.gridConfig?.enableSearch\">\n                    <ngx-search (onSearch)=\"handleSearch($event)\">\n                        <ng-content select=\"[searchContent]\"></ng-content>\n                    </ngx-search>\n                </ng-container>\n\n                <!--Ngx Table-->\n                <ngx-primeng-basic-grid [key]=\"_layout?.setting?.gridConfig?.pk\" [columns]=\"_layout?.columns\"\n                    [result]=\"_result\" [filter]=\"_filter\" (onFilterChanged)=\"handleFilterChanged($event)\"\n                    [loading]=\"_loading\">\n\n                    <!--Custom column-->\n                    <ng-template gridTemplate=\"custom\" let-row=\"row\" let-column=\"column\">\n                        <!--Custom column-->\n                        <ng-container *ngTemplateOutlet=\"_customColumnTemplate;context:{row:row,column:column}\">\n                        </ng-container>\n\n                        <!--For action column-->\n                        <ng-container *ngIf=\"column.key=='action'\">\n                            <!--Edit-->\n                            <button pButton pRipple type=\"button\" icon=\"pi pi-pencil\"\n                                class=\"p-button-rounded p-button-text p-button-sm mr-2\" *ngIf=\"row?.action?.edit\"\n                                (click)=\"onActionEdit(row)\"></button>\n                            <!--Delete-->\n                            <button pButton pRipple type=\"button\" icon=\"pi pi-trash\"\n                                class=\"p-button-rounded p-button-text p-button-sm p-button-danger\"\n                                *ngIf=\"row?.action?.delete\" (click)=\"onActionDelete(row)\"></button>\n                        </ng-container>\n\n                    </ng-template>\n\n                </ngx-primeng-basic-grid>\n            </div>\n        </div>\n    </div>\n    <!--Footer content-->\n    <ng-content select=[footer]></ng-content>\n</div>\n\n<!--Confirm dialog-->\n<p-confirmDialog appendTo=\"body\" [key]=\"_dialogKey\"></p-confirmDialog>",
                styles: [""]
            },] }
];
NgxCrudGridComponent.ctorParameters = () => [
    { type: DialogService },
    { type: ConfirmationService }
];
NgxCrudGridComponent.propDecorators = {
    templates: [{ type: ContentChildren, args: [GridCustomTemplate,] }],
    layout: [{ type: Input }],
    breadcrumbs: [{ type: Input }],
    onReady: [{ type: Output }]
};

class NgxCrudLayoutModule {
}
NgxCrudLayoutModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    NgxCrudManageComponent,
                    NgxCrudManageDialogComponent,
                    NgxCrudManageLayoutComponent,
                    NgxCrudGridComponent
                ],
                entryComponents: [NgxCrudManageDialogComponent],
                imports: [
                    CommonModule,
                    FormsModule,
                    ReactiveFormsModule,
                    NgxPrimengGridModule,
                    NgxWrapperSharedModule,
                    ButtonModule,
                    NgxPrimengFormModule,
                    DynamicDialogModule,
                    ConfirmDialogModule
                ],
                providers: [
                    DialogService,
                    ConfirmationService
                ],
                exports: [NgxCrudGridComponent, NgxCrudManageLayoutComponent, NgxCrudManageComponent]
            },] }
];

class NgxCrudService {
    constructor() { }
    prepareLayout(rawLayout) {
        if (rawLayout) {
            const layout = {
                api: this.prepareApiConfig(rawLayout),
                columns: _.has(rawLayout, 'columns') ? rawLayout.columns : [],
                form: this.prepareFormConfig(rawLayout),
                setting: this.prepareSettingConfig(rawLayout)
            };
            return layout;
        }
        return null;
    }
    prepareFormRoute(activatedRoute, callback) {
        // set config
        const config = activatedRoute.snapshot.data['config'];
        if (config) {
            // set layout key
            config['layoutKey'] = _.has(config, 'layoutKey') ? config.layoutKey : 'layout';
            // get layout object
            const rawLayout = activatedRoute.snapshot.data[config.layoutKey];
            if (rawLayout) {
                // prepare layout
                const layout = this.prepareLayout(rawLayout);
                // call back
                callback(config, layout);
            }
        }
    }
    // prepare api
    prepareApiConfig(data) {
        if (_.has(data, 'api')) {
            const api = data.api;
            const apiConfig = {
                changeStatus: _.has(api, 'changeStatus') ? api.changeStatus : null,
                delete: _.has(api, 'delete') ? api.delete : null,
                manage: _.has(api, 'manage') ? api.manage : null,
                search: _.has(api, 'search') ? api.search : null,
                command: (path) => { return path; }
            };
            return apiConfig;
        }
        return null;
    }
    // prepare setting
    prepareSettingConfig(data) {
        // setting layout
        const settingLayout = {};
        // check setting attribute is exist or not
        if (_.has(data, 'setting')) {
            // setting instance
            const setting = data.setting;
            // title
            settingLayout.pageTitle = _.has(setting, 'pageTitle') ? setting.pageTitle : 'NA';
            // prepare manage Config
            this.prepareManageConfigSetting(setting, settingLayout);
            // prepare grid config
            this.prepareGridConfigSetting(setting, settingLayout);
        }
        return settingLayout;
    }
    prepareManageConfigSetting(setting, settingLayout) {
        if (_.has(setting, 'actionConfig')) {
            // action config
            const actionConfig = setting.actionConfig;
            // init config
            settingLayout.actionConfig = { actions: [], enable: false };
            // check manage is enabled or not
            settingLayout.actionConfig.enable = _.has(actionConfig, 'enable') ? actionConfig.enable : false;
            if (settingLayout.actionConfig.enable) {
                // get actions
                const actions = _.has(actionConfig, 'actions') ? actionConfig.actions : [];
                // action Null check
                if (actions && actions.length > 0) {
                    // loop action info
                    actions.forEach(info => {
                        // action instance
                        const actionInfo = {};
                        // set key
                        actionInfo.key = _.has(info, 'key') ? info.key : null;
                        // set label
                        actionInfo.label = _.has(info, 'label') ? info.label : null;
                        // set icon
                        actionInfo.icon = _.has(info, 'icon') ? info.icon : null;
                        // set styleClass
                        actionInfo.styleClass = _.has(info, 'styleClass') ? info.styleClass : 'p-button-sm p-button-outlined';
                        // set mode
                        actionInfo.mode = _.has(info, 'mode') ? info.mode : null;
                        // check for modal config
                        if (_.has(info, 'modalConfig')) {
                            actionInfo.modalConfig = {
                                title: _.has(info, 'modalConfig.title') ? info.modalConfig.title : null,
                                width: _.has(info, 'modalConfig.width') ? info.modalConfig.width : 'auto'
                            };
                        }
                        // push
                        settingLayout.actionConfig.actions.push(actionInfo);
                    });
                }
            }
            // for action
            settingLayout.actionConfig.onAction = (action) => {
                // sudo action
            };
        }
    }
    prepareGridConfigSetting(setting, settingLayout) {
        if (_.has(setting, 'gridConfig')) {
            // grid config instance
            const gridConfig = setting.gridConfig;
            // init config
            settingLayout.gridConfig = {};
            // set pk
            settingLayout.gridConfig.pk = _.has(gridConfig, 'pk') ? gridConfig.pk : null;
            // set layoutId
            settingLayout.gridConfig.layoutId = _.has(gridConfig, 'layoutId') ? gridConfig.layoutId : null;
            // set enableSearch
            settingLayout.gridConfig.enableSearch = _.has(gridConfig, 'enableSearch') ? gridConfig.enableSearch : false;
            // set sortField
            settingLayout.gridConfig.sortField = _.has(gridConfig, 'sortField') ? gridConfig.sortField : null;
            // set sortOrder
            settingLayout.gridConfig.sortOrder = _.has(gridConfig, 'sortOrder') ? gridConfig.sortOrder : -1;
            // set sortOrder
            settingLayout.gridConfig.perPage = _.has(gridConfig, 'perPage') ? gridConfig.perPage : 10;
            // set breadcrumbs
            settingLayout.gridConfig.breadcrumbs = _.has(gridConfig, 'breadcrumbs') ? gridConfig.breadcrumbs : [];
            // action
            // sudo action
            settingLayout.gridConfig.action = {
                customAction: {},
                onFilterChanged: (filter) => { },
                loadResult: () => { }
            };
        }
    }
    prepareFormConfig(data) {
        // init form form config
        const formConfig = {};
        // check form attribute is exist or not
        if (_.has(data, 'form')) {
            // form instance
            const form = data.form;
            // check control
            if (_.has(form, 'controls')) {
                // set controls
                formConfig.controls = form.controls;
                // prepare action
                // sudo function
                formConfig.action = {
                    onExecute: (formValue, action, data, callback) => { },
                    onPrepareValue: (formgroup, formItems, data) => { },
                    onPatchValue: (formgroup, formItems, data) => { }
                };
            }
        }
        return formConfig;
    }
}
NgxCrudService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgxCrudService_Factory() { return new NgxCrudService(); }, token: NgxCrudService, providedIn: "root" });
NgxCrudService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NgxCrudService.ctorParameters = () => [];

/*
 * Public API Surface of ngx-wrapper
 */

/**
 * Generated bundle index. Do not edit.
 */

export { BaseComponent, DialogBaseComponent, DialogFormBaseComponent, DynamicFormBaseComponent, FormBaseComponent, ListViewDataType, NgxCardLayoutComponent, NgxCrudLayoutModule, NgxCrudLayoutSetting_ManageConfig_Mode, NgxCrudService, NgxHeaderComponent, NgxInlineLoaderComponent, NgxLambdaUtility, NgxListDataViewComponent, NgxManageActionLayoutComponent, NgxManageLayoutBaseComponent, NgxManageLayoutComponent, NgxManageLayoutDialogBaseComponent, NgxManageLayoutModule, NgxManageLayoutService, NgxSearchComponent, NgxUtility, NgxValidationService, NgxWrapperModule, NgxWrapperSharedModule, NgxFullpageLoaderComponent as ɵa, NgxCrudManageComponent as ɵb, NgxCrudManageDialogComponent as ɵc, NgxCrudManageLayoutComponent as ɵd, NgxCrudGridComponent as ɵe };
//# sourceMappingURL=ngx-wrapper.js.map
